@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Mauro Carvalho Chehab <mchehab@redhat.com>
+ * Copyright (C) 2013 Mauro Carvalho Chehab <mchehab+redhat@kernel.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,16 +26,24 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/poll.h>
-#include "libtrace/kbuffer.h"
-#include "libtrace/event-parse.h"
+#include <signal.h>
+#include <sys/signalfd.h>
+#include <linux/version.h>
+#include <traceevent/kbuffer.h>
+#include <traceevent/event-parse.h>
 #include "ras-mc-handler.h"
 #include "ras-aer-handler.h"
 #include "ras-non-standard-handler.h"
 #include "ras-arm-handler.h"
 #include "ras-mce-handler.h"
 #include "ras-extlog-handler.h"
+#include "ras-devlink-handler.h"
+#include "ras-diskerror-handler.h"
+#include "ras-memory-failure-handler.h"
 #include "ras-record.h"
 #include "ras-logger.h"
+#include "ras-page-isolation.h"
+#include "ras-cpu-isolation.h"
 
 /*
  * Polling time, if read() doesn't block. Currently, trace_pipe_raw never
@@ -218,17 +227,66 @@ int toggle_ras_mc_event(int enable)
 	rc |= __toggle_ras_mc_event(ras, "ras", "arm_event", enable);
 #endif
 
+#ifdef HAVE_DEVLINK
+	rc |= __toggle_ras_mc_event(ras, "devlink", "devlink_health_report", enable);
+#endif
+
+#ifdef HAVE_DISKERROR
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,18,0)
+	rc |= __toggle_ras_mc_event(ras, "block", "block_rq_error", enable);
+#else
+	rc |= __toggle_ras_mc_event(ras, "block", "block_rq_complete", enable);
+#endif
+#endif
+
+#ifdef HAVE_MEMORY_FAILURE
+	rc |= __toggle_ras_mc_event(ras, "ras", "memory_failure_event", enable);
+#endif
+
 free_ras:
 	free(ras);
 	return rc;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5,18,0)
+/*
+ * Set kernel filter. libtrace doesn't provide an API for setting filters
+ * in kernel, we have to implement it here.
+ */
+static int filter_ras_mc_event(struct ras_events *ras, char *group, char *event,
+			       const char *filter_str)
+{
+	int fd, rc;
+	char fname[MAX_PATH + 1];
+
+	snprintf(fname, sizeof(fname), "events/%s/%s/filter", group, event);
+	fd = open_trace(ras, fname, O_RDWR | O_APPEND);
+	if (fd < 0) {
+		log(ALL, LOG_WARNING, "Can't open filter file\n");
+		return errno;
+	}
+
+	rc = write(fd, filter_str ,strlen(filter_str));
+	if (rc < 0) {
+		log(ALL, LOG_WARNING, "Can't write to filter file\n");
+		close(fd);
+		return rc;
+	}
+	close(fd);
+	if (!rc) {
+		log(ALL, LOG_WARNING, "Nothing was written on filter file\n");
+		return EIO;
+	}
+
+	return 0;
+}
+#endif
 
 /*
  * Tracing read code
  */
 
-static int get_pagesize(struct ras_events *ras, struct pevent *pevent)
+static int get_pagesize(struct ras_events *ras, struct tep_handle *pevent)
 {
 	int fd, len, page_size = 4096;
 	char buf[page_size];
@@ -240,11 +298,9 @@ static int get_pagesize(struct ras_events *ras, struct pevent *pevent)
 	len = read(fd, buf, page_size);
 	if (len <= 0)
 		goto error;
-	if (pevent_parse_header_page(pevent, buf, len, sizeof(long)))
+	if (tep_parse_header_page(pevent, buf, len, sizeof(long)))
 		goto error;
 
-	page_size = pevent->header_page_data_offset + pevent->header_page_data_size;
-
 error:
 	close(fd);
 	return page_size;
@@ -254,13 +310,14 @@ error:
 static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 			   void *data, unsigned long long time_stamp)
 {
-	struct pevent_record record;
+	struct tep_record record;
 	struct trace_seq s;
 
 	record.ts = time_stamp;
 	record.size = kbuffer_event_size(kbuf);
 	record.data = data;
 	record.offset = kbuffer_curr_offset(kbuf);
+	record.cpu = pdata->cpu;
 
 	/* note offset is just offset in subbuffer */
 	record.missed_events = kbuffer_missed_events(kbuf);
@@ -268,11 +325,15 @@ static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 
 	/* TODO - logging */
 	trace_seq_init(&s);
-	printf("cpu %02d:", pdata->cpu);
-	fflush(stdout);
-	pevent_print_event(pdata->ras->pevent, &s, &record);
+	tep_print_event(pdata->ras->pevent, &s, &record,
+			"%16s-%-5d [%03d] %s %6.1000d %s %s",
+                        TEP_PRINT_COMM, TEP_PRINT_PID, TEP_PRINT_CPU,
+                        TEP_PRINT_LATENCY, TEP_PRINT_TIME, TEP_PRINT_NAME,
+                        TEP_PRINT_INFO);
 	trace_seq_do_printf(&s);
 	printf("\n");
+	fflush(stdout);
+	trace_seq_destroy(&s);
 }
 
 static int get_num_cpus(struct ras_events *ras)
@@ -309,9 +370,14 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	int ready, i, count_nready;
 	struct kbuffer *kbuf;
 	void *page;
-	struct pollfd fds[n_cpus];
+	struct pollfd fds[n_cpus + 1];
+	struct signalfd_siginfo fdsiginfo;
+	sigset_t mask;
 	int warnonce[n_cpus];
 	char pipe_raw[PATH_MAX];
+	int legacy_kernel = 0;
+	int fd;
+	char buf[16];
 #if 0
 	int need_sleep = 0;
 #endif
@@ -331,6 +397,29 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 		return -ENOMEM;
 	}
 
+	/* Fix for poll() on the per_cpu trace_pipe and trace_pipe_raw blocks
+	 * indefinitely with the default buffer_percent in the kernel trace system,
+	 * which is introduced by the following change in the kernel.
+	 * https://lore.kernel.org/all/20221020231427.41be3f26@gandalf.local.home/T/#u.
+	 * Set buffer_percent to 0 so that poll() will return immediately
+	 * when the trace data is available in the ras per_cpu trace pipe_raw
+	 */
+	fd = open_trace(pdata[0].ras, "buffer_percent", O_WRONLY);
+	if (fd >= 0) {
+		/* For the backward compatibility to the old kernels, do not return
+		 * if fail to set the buffer_percent.
+		 */
+		snprintf(buf, sizeof(buf), "0");
+		size = write(fd, buf, strlen(buf));
+		if (size <= 0)
+			log(TERM, LOG_WARNING, "can't write to buffer_percent\n");
+		close(fd);
+	} else
+		log(TERM, LOG_WARNING, "Can't open buffer_percent\n");
+
+	for (i = 0; i < (n_cpus + 1); i++)
+		fds[i].fd = -1;
+
 	for (i = 0; i < n_cpus; i++) {
 		fds[i].events = POLLIN;
 
@@ -341,21 +430,57 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 		fds[i].fd = open_trace(pdata[0].ras, pipe_raw, O_RDONLY);
 		if (fds[i].fd < 0) {
 			log(TERM, LOG_ERR, "Can't open trace_pipe_raw\n");
-			kbuffer_free(kbuf);
-			free(page);
-			return -EINVAL;
+			goto error;
 		}
 	}
 
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGINT);
+	sigaddset(&mask, SIGTERM);
+	sigaddset(&mask, SIGHUP);
+	sigaddset(&mask, SIGQUIT);
+	if (sigprocmask(SIG_BLOCK, &mask, NULL) == -1)
+		log(TERM, LOG_WARNING, "sigprocmask\n");
+	fds[n_cpus].events = POLLIN;
+	fds[n_cpus].fd = signalfd(-1, &mask, 0);
+	if (fds[n_cpus].fd < 0) {
+		log(TERM, LOG_WARNING, "signalfd\n");
+		goto error;
+	}
+
 	log(TERM, LOG_INFO, "Listening to events for cpus 0 to %d\n", n_cpus - 1);
-	if (pdata[0].ras->record_events)
-		ras_mc_event_opendb(pdata[0].cpu, pdata[0].ras);
+	if (pdata[0].ras->record_events) {
+		if (ras_mc_event_opendb(pdata[0].cpu, pdata[0].ras))
+			goto error;
+	}
 
 	do {
-		ready = poll(fds, n_cpus, -1);
+		ready = poll(fds, (n_cpus + 1), -1);
 		if (ready < 0) {
 			log(TERM, LOG_WARNING, "poll\n");
 		}
+
+		/* check for the signal */
+		if (fds[n_cpus].revents & POLLIN) {
+			size = read(fds[n_cpus].fd, &fdsiginfo,
+				    sizeof(struct signalfd_siginfo));
+			if (size != sizeof(struct signalfd_siginfo))
+				log(TERM, LOG_WARNING, "signalfd read\n");
+
+			if (fdsiginfo.ssi_signo == SIGINT ||
+			    fdsiginfo.ssi_signo == SIGTERM ||
+			    fdsiginfo.ssi_signo == SIGHUP ||
+			    fdsiginfo.ssi_signo == SIGQUIT) {
+				log(TERM, LOG_INFO, "Recevied signal=%d\n",
+				    fdsiginfo.ssi_signo);
+				goto  cleanup;
+			} else {
+				log(TERM, LOG_INFO,
+				    "Received unexpected signal=%d\n",
+				    fdsiginfo.ssi_signo);
+			}
+		}
+
 		count_nready = 0;
 		for (i = 0; i < n_cpus; i++) {
 			if (fds[i].revents & POLLERR) {
@@ -375,7 +500,7 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 			size = read(fds[i].fd, page, pdata[i].ras->page_size);
 			if (size < 0) {
 				log(TERM, LOG_WARNING, "read\n");
-				return -1;
+				goto cleanup;
 			} else if (size > 0) {
 				kbuffer_load_subbuffer(kbuf, page);
 
@@ -400,6 +525,7 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 		 */
 		if (count_nready == n_cpus) {
 			/* Should only happen with legacy kernels */
+			legacy_kernel = 1;
 			break;
 		}
 #endif
@@ -408,12 +534,25 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	/* poll() is not supported. We need to fallback to the old way */
 	log(TERM, LOG_INFO,
 	    "Old kernel detected. Stop listening and fall back to pthread way.\n");
+
+cleanup:
+	if (pdata[0].ras->record_events)
+		ras_mc_event_closedb(pdata[0].cpu, pdata[0].ras);
+
+error:
 	kbuffer_free(kbuf);
 	free(page);
-	for (i = 0; i < n_cpus; i++)
-		close(fds[i].fd);
+	sigprocmask(SIG_UNBLOCK, &mask, NULL);
 
-	return -255;
+	for (i = 0; i < (n_cpus + 1); i++) {
+		if (fds[i].fd > 0)
+			close(fds[i].fd);
+	}
+
+	if (legacy_kernel)
+		return -255;
+	else
+		return -1;
 }
 
 static int read_ras_event(int fd,
@@ -421,7 +560,7 @@ static int read_ras_event(int fd,
 			  struct kbuffer *kbuf,
 			  void *page)
 {
-	unsigned size;
+	int size;
 	unsigned long long time_stamp;
 	void *data;
 
@@ -485,11 +624,21 @@ static void *handle_ras_events_cpu(void *priv)
 	}
 
 	log(TERM, LOG_INFO, "Listening to events on cpu %d\n", pdata->cpu);
-	if (pdata->ras->record_events)
-		ras_mc_event_opendb(pdata->cpu, pdata->ras);
+	if (pdata->ras->record_events) {
+		if (ras_mc_event_opendb(pdata->cpu, pdata->ras)) {
+			log(TERM, LOG_ERR, "Can't open database\n");
+			close(fd);
+			kbuffer_free(kbuf);
+			free(page);
+			return 0;
+		}
+	}
 
 	read_ras_event(fd, pdata, kbuf, page);
 
+	if (pdata->ras->record_events)
+		ras_mc_event_closedb(pdata->cpu, pdata->ras);
+
 	close(fd);
 	kbuffer_free(kbuf);
 	free(page);
@@ -550,12 +699,12 @@ static int select_tracing_timestamp(struct ras_events *ras)
 		return 0;
 	}
 	rc = fscanf(fp, "%zu.%u ", &uptime, &j1);
+	fclose(fp);
 	if (rc <= 0) {
 		log(TERM, LOG_ERR, "Can't parse /proc/uptime!\n");
 		return -1;
 	}
 	now = time(NULL);
-	fclose(fp);
 
 	ras->use_uptime = 1;
 	ras->uptime_diff = now - uptime;
@@ -563,12 +712,13 @@ static int select_tracing_timestamp(struct ras_events *ras)
 	return 0;
 }
 
-static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
+static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
 			     unsigned page_size, char *group, char *event,
-			     pevent_event_handler_func func)
+			     tep_event_handler_func func, char *filter_str, int id)
 {
 	int fd, size, rc;
 	char *page, fname[MAX_PATH + 1];
+	struct tep_event_filter * filter = NULL;
 
 	snprintf(fname, sizeof(fname), "events/%s/%s/format", group, event);
 
@@ -598,23 +748,47 @@ static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
 	}
 
 	/* Registers the special event handlers */
-	rc = pevent_register_event_handler(pevent, -1, group, event, func, ras);
-	if (rc == PEVENT_ERRNO__MEM_ALLOC_FAILED) {
+	rc = tep_register_event_handler(pevent, -1, group, event, func, ras);
+	if (rc == TEP_ERRNO__MEM_ALLOC_FAILED) {
 		log(TERM, LOG_ERR, "Can't register event handler for %s:%s\n",
 		    group, event);
 		free(page);
 		return EINVAL;
 	}
 
-	rc = pevent_parse_event(pevent, page, size, group);
+	rc = tep_parse_event(pevent, page, size, group);
 	if (rc) {
 		log(TERM, LOG_ERR, "Can't parse event %s:%s\n", group, event);
 		free(page);
 		return EINVAL;
 	}
 
+	if (filter_str) {
+		char error[255];
+
+		filter = tep_filter_alloc(pevent);
+		if (!filter) {
+			log(TERM, LOG_ERR,
+			    "Failed to allocate filter for %s/%s.\n", group, event);
+			free(page);
+			return EINVAL;
+		}
+		rc = tep_filter_add_filter_str(filter, filter_str);
+		if (rc < 0) {
+			tep_filter_strerror(filter, rc, error, sizeof(error));
+			log(TERM, LOG_ERR,
+			    "Failed to install filter for %s/%s: %s\n", group, event, error);
+			tep_filter_free(filter);
+			free(page);
+			return rc;
+		}
+	}
+
+	ras->filters[id] = filter;
+
 	/* Enable RAS events */
 	rc = __toggle_ras_mc_event(ras, group, event, 1);
+	free(page);
 	if (rc < 0) {
 		log(TERM, LOG_ERR, "Can't enable %s:%s tracing\n",
 		    group, event);
@@ -624,7 +798,6 @@ static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
 
 	log(ALL, LOG_INFO, "Enabled event %s:%s\n", group, event);
 
-	free(page);
 	return 0;
 }
 
@@ -633,9 +806,12 @@ int handle_ras_events(int record_events)
 	int rc, page_size, i;
 	int num_events = 0;
 	unsigned cpus;
-	struct pevent *pevent = NULL;
+	struct tep_handle *pevent = NULL;
 	struct pthread_data *data = NULL;
 	struct ras_events *ras = NULL;
+#ifdef HAVE_DEVLINK
+	char *filter_str = NULL;
+#endif
 
 	ras = calloc(1, sizeof(*ras));
 	if (!ras) {
@@ -655,7 +831,7 @@ int handle_ras_events(int record_events)
 		goto err;
 	}
 
-	pevent = pevent_alloc();
+	pevent = tep_alloc();
 	if (!pevent) {
 		log(TERM, LOG_ERR, "Can't allocate pevent\n");
 		rc = errno;
@@ -668,8 +844,13 @@ int handle_ras_events(int record_events)
 	ras->page_size = page_size;
 	ras->record_events = record_events;
 
+#ifdef HAVE_MEMORY_CE_PFA
+	/* FIXME: enable memory isolation unconditionally */
+	ras_page_account_init();
+#endif
+
 	rc = add_event_handler(ras, pevent, page_size, "ras", "mc_event",
-			       ras_mc_event_handler);
+			       ras_mc_event_handler, NULL, MC_EVENT);
 	if (!rc)
 		num_events++;
 	else
@@ -678,7 +859,7 @@ int handle_ras_events(int record_events)
 
 #ifdef HAVE_AER
 	rc = add_event_handler(ras, pevent, page_size, "ras", "aer_event",
-			       ras_aer_event_handler);
+			       ras_aer_event_handler, NULL, AER_EVENT);
 	if (!rc)
 		num_events++;
 	else
@@ -687,27 +868,31 @@ int handle_ras_events(int record_events)
 #endif
 
 #ifdef HAVE_NON_STANDARD
-        rc = add_event_handler(ras, pevent, page_size, "ras", "non_standard_event",
-                               ras_non_standard_event_handler);
-        if (!rc)
-                num_events++;
-        else
-                log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
-                    "ras", "non_standard_event");
+	rc = add_event_handler(ras, pevent, page_size, "ras", "non_standard_event",
+			       ras_non_standard_event_handler, NULL, NON_STANDARD_EVENT);
+	if (!rc)
+		num_events++;
+	else
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "ras", "non_standard_event");
 #endif
 
 #ifdef HAVE_ARM
-        rc = add_event_handler(ras, pevent, page_size, "ras", "arm_event",
-                               ras_arm_event_handler);
-        if (!rc)
-                num_events++;
-        else
-                log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
-                    "ras", "arm_event");
+	rc = add_event_handler(ras, pevent, page_size, "ras", "arm_event",
+			       ras_arm_event_handler, NULL, ARM_EVENT);
+	if (!rc)
+		num_events++;
+	else
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "ras", "arm_event");
 #endif
 
 	cpus = get_num_cpus(ras);
 
+#ifdef HAVE_CPU_FAULT_ISOLATION
+	ras_cpu_isolation_init(cpus);
+#endif
+
 #ifdef HAVE_MCE
 	rc = register_mce_handler(ras, cpus);
 	if (rc)
@@ -715,7 +900,7 @@ int handle_ras_events(int record_events)
 	if (ras->mce_priv) {
 		rc = add_event_handler(ras, pevent, page_size,
 				       "mce", "mce_record",
-			               ras_mce_event_handler);
+				       ras_mce_event_handler, NULL, MCE_EVENT);
 		if (!rc)
 			num_events++;
 	else
@@ -726,20 +911,73 @@ int handle_ras_events(int record_events)
 
 #ifdef HAVE_EXTLOG
 	rc = add_event_handler(ras, pevent, page_size, "ras", "extlog_mem_event",
-			       ras_extlog_mem_event_handler);
+			       ras_extlog_mem_event_handler, NULL, EXTLOG_EVENT);
 	if (!rc) {
 		/* tell kernel we are listening, so don't printk to console */
 		(void)open("/sys/kernel/debug/ras/daemon_active", 0);
 		num_events++;
 	} else
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
-		    "ras", "aer_event");
+		    "ras", "extlog_mem_event");
+#endif
+
+#ifdef HAVE_DEVLINK
+	rc = add_event_handler(ras, pevent, page_size, "net",
+			       "net_dev_xmit_timeout",
+			       ras_net_xmit_timeout_handler, NULL, DEVLINK_EVENT);
+	if (!rc)
+		filter_str = "devlink/devlink_health_report:msg=~\'TX timeout*\'";
+
+	rc = add_event_handler(ras, pevent, page_size, "devlink",
+			       "devlink_health_report",
+			       ras_devlink_event_handler, filter_str, DEVLINK_EVENT);
+	if (!rc)
+		num_events++;
+	else
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "devlink", "devlink_health_report");
+#endif
+
+#ifdef HAVE_DISKERROR
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,18,0)
+	rc = add_event_handler(ras, pevent, page_size, "block",
+			       "block_rq_error", ras_diskerror_event_handler,
+				NULL, DISKERROR_EVENT);
+	if (!rc)
+		num_events++;
+	else
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "block", "block_rq_error");
+#else
+	rc = filter_ras_mc_event(ras, "block", "block_rq_complete", "error != 0");
+	if (!rc) {
+		rc = add_event_handler(ras, pevent, page_size, "block",
+				       "block_rq_complete", ras_diskerror_event_handler,
+					NULL, DISKERROR_EVENT);
+		if (!rc)
+			num_events++;
+		else
+			log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+			    "block", "block_rq_complete");
+	}
+#endif
+#endif
+
+#ifdef HAVE_MEMORY_FAILURE
+	rc = add_event_handler(ras, pevent, page_size, "ras", "memory_failure_event",
+			       ras_memory_failure_event_handler, NULL, MF_EVENT);
+	if (!rc)
+		num_events++;
+	else
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "ras", "memory_failure_event");
 #endif
 
 	if (!num_events) {
 		log(ALL, LOG_INFO,
 		    "Failed to trace all supported RAS events. Aborting.\n");
-		return EINVAL;
+		rc = -EINVAL;
+		goto err;
 	}
 
 	data = calloc(sizeof(*data), cpus);
@@ -783,10 +1021,17 @@ err:
 		free(data);
 
 	if (pevent)
-		pevent_free(pevent);
+		tep_free(pevent);
 
-	if (ras)
+	if (ras) {
+		for (i = 0; i < NR_EVENTS; i++) {
+			if (ras->filters[i])
+				tep_filter_free(ras->filters[i]);
+		}
 		free(ras);
-
+	}
+#ifdef HAVE_CPU_FAULT_ISOLATION
+	cpu_infos_free();
+#endif
 	return rc;
 }
