@@ -19,7 +19,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "libtrace/kbuffer.h"
+#include <traceevent/kbuffer.h>
 #include "ras-mc-handler.h"
 #include "ras-record.h"
 #include "ras-logger.h"
@@ -27,8 +27,8 @@
 #include "ras-report.h"
 
 int ras_mc_event_handler(struct trace_seq *s,
-			 struct pevent_record *record,
-			 struct event_format *event, void *context)
+			 struct tep_record *record,
+			 struct tep_event *event, void *context)
 {
 	int len;
 	unsigned long long val;
@@ -58,14 +58,14 @@ int ras_mc_event_handler(struct trace_seq *s,
 			 "%Y-%m-%d %H:%M:%S %z", tm);
 	trace_seq_printf(s, "%s ", ev.timestamp);
 
-	if (pevent_get_field_val(s,  event, "error_count", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "error_count", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 
 	ev.error_count = val;
 	trace_seq_printf(s, "%d ", ev.error_count);
 
-	if (pevent_get_field_val(s, event, "error_type", record, &val, 1) < 0)
+	if (tep_get_field_val(s, event, "error_type", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 
@@ -90,7 +90,7 @@ int ras_mc_event_handler(struct trace_seq *s,
 	else
 		trace_seq_puts(s, " error:");
 
-	ev.msg = pevent_get_field_raw(s, event, "msg", record, &len, 1);
+	ev.msg = tep_get_field_raw(s, event, "msg", record, &len, 1);
 	if (!ev.msg)
 		goto parse_error;
 	parsed_fields++;
@@ -100,7 +100,7 @@ int ras_mc_event_handler(struct trace_seq *s,
 		trace_seq_puts(s, ev.msg);
 	}
 
-	ev.label = pevent_get_field_raw(s, event, "label", record, &len, 1);
+	ev.label = tep_get_field_raw(s, event, "label", record, &len, 1);
 	if (!ev.label)
 		goto parse_error;
 	parsed_fields++;
@@ -111,24 +111,24 @@ int ras_mc_event_handler(struct trace_seq *s,
 	}
 
 	trace_seq_puts(s, " (");
-	if (pevent_get_field_val(s,  event, "mc_index", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "mc_index", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 
 	ev.mc_index = val;
 	trace_seq_printf(s, "mc: %d", ev.mc_index);
 
-	if (pevent_get_field_val(s,  event, "top_layer", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "top_layer", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 	ev.top_layer = (signed char) val;
 
-	if (pevent_get_field_val(s,  event, "middle_layer", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "middle_layer", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 	ev.middle_layer = (signed char) val;
 
-	if (pevent_get_field_val(s,  event, "lower_layer", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "lower_layer", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 	ev.lower_layer = (signed char) val;
@@ -144,7 +144,7 @@ int ras_mc_event_handler(struct trace_seq *s,
 			trace_seq_printf(s, " location: %d", ev.top_layer);
 	}
 
-	if (pevent_get_field_val(s,  event, "address", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "address", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 
@@ -152,7 +152,7 @@ int ras_mc_event_handler(struct trace_seq *s,
 	if (ev.address)
 		trace_seq_printf(s, " address: 0x%08llx", ev.address);
 
-	if (pevent_get_field_val(s,  event, "grain_bits", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "grain_bits", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 
@@ -160,7 +160,7 @@ int ras_mc_event_handler(struct trace_seq *s,
 	trace_seq_printf(s, " grain: %lld", ev.grain);
 
 
-	if (pevent_get_field_val(s,  event, "syndrome", record, &val, 1) < 0)
+	if (tep_get_field_val(s,  event, "syndrome", record, &val, 1) < 0)
 		goto parse_error;
 	parsed_fields++;
 
@@ -168,7 +168,7 @@ int ras_mc_event_handler(struct trace_seq *s,
 	if (val)
 		trace_seq_printf(s, " syndrome: 0x%08llx", ev.syndrome);
 
-	ev.driver_detail = pevent_get_field_raw(s, event, "driver_detail", record,
+	ev.driver_detail = tep_get_field_raw(s, event, "driver_detail", record,
 					     &len, 1);
 	if (!ev.driver_detail)
 		goto parse_error;
