@@ -22,7 +22,7 @@
 #include <stdint.h>
 
 #include "ras-events.h"
-#include "libtrace/event-parse.h"
+#include <traceevent/event-parse.h>
 
 enum cputype {
 	CPU_GENERIC,
@@ -112,8 +112,8 @@ struct mce_priv {
 /* register and handling routines */
 int register_mce_handler(struct ras_events *ras, unsigned ncpus);
 int ras_mce_event_handler(struct trace_seq *s,
-			  struct pevent_record *record,
-			  struct event_format *event, void *context);
+			  struct tep_record *record,
+			  struct tep_event *event, void *context);
 
 /* enables intel iMC logs */
 int set_intel_imc_log(enum cputype cputype, unsigned ncpus);
