@@ -33,10 +33,12 @@ static int setup_report_socket(void){
 
 	memset(&addr, 0, sizeof(struct sockaddr_un));
 	addr.sun_family = AF_UNIX;
-	strncpy(addr.sun_path, ABRT_SOCKET, strlen(ABRT_SOCKET));
+	strncpy(addr.sun_path, ABRT_SOCKET, sizeof(addr.sun_path));
+	addr.sun_path[sizeof(addr.sun_path) - 1] = '\0';
 
 	rc = connect(sockfd, (struct sockaddr *)&addr, sizeof(struct sockaddr_un));
-	if (rc < 0){
+	if (rc < 0) {
+		close(sockfd);
 		return -1;
 	}
 
@@ -255,6 +257,80 @@ static int set_arm_event_backtrace(char *buf, struct ras_arm_event *ev){
 	return 0;
 }
 
+static int set_devlink_event_backtrace(char *buf, struct devlink_event *ev){
+	char bt_buf[MAX_BACKTRACE_SIZE];
+
+	if(!buf || !ev)
+		return -1;
+
+	sprintf(bt_buf, "BACKTRACE="	\
+						"timestamp=%s\n"	\
+						"bus_name=%s\n"		\
+						"dev_name=%s\n"		\
+						"driver_name=%s\n"	\
+						"reporter_name=%s\n"	\
+						"msg=%s\n",		\
+						ev->timestamp,		\
+						ev->bus_name,		\
+						ev->dev_name,		\
+						ev->driver_name,	\
+						ev->reporter_name,	\
+						ev->msg);
+
+	strcat(buf, bt_buf);
+
+	return 0;
+}
+
+static int set_diskerror_event_backtrace(char *buf, struct diskerror_event *ev) {
+	char bt_buf[MAX_BACKTRACE_SIZE];
+
+	if(!buf || !ev)
+		return -1;
+
+	sprintf(bt_buf, "BACKTRACE="	\
+						"timestamp=%s\n"	\
+						"dev=%s\n"		\
+						"sector=%llu\n"		\
+						"nr_sector=%u\n"	\
+						"error=%s\n"		\
+						"rwbs=%s\n"		\
+						"cmd=%s\n",		\
+						ev->timestamp,		\
+						ev->dev,		\
+						ev->sector,		\
+						ev->nr_sector,		\
+						ev->error,		\
+						ev->rwbs,		\
+						ev->cmd);
+
+	strcat(buf, bt_buf);
+
+	return 0;
+}
+
+static int set_mf_event_backtrace(char *buf, struct ras_mf_event *ev)
+{
+	char bt_buf[MAX_BACKTRACE_SIZE];
+
+	if (!buf || !ev)
+		return -1;
+
+	sprintf(bt_buf, "BACKTRACE="    \
+                                                "timestamp=%s\n"	\
+                                                "pfn=%s\n"		\
+                                                "page_type=%s\n"	\
+                                                "action_result=%s\n",	\
+                                                ev->timestamp,		\
+                                                ev->pfn,		\
+                                                ev->page_type,		\
+                                                ev->action_result);
+
+	strcat(buf, bt_buf);
+
+	return 0;
+}
+
 static int commit_report_backtrace(int sockfd, int type, void *ev){
 	char buf[MAX_BACKTRACE_SIZE];
 	char *pbuf = buf;
@@ -283,6 +359,15 @@ static int commit_report_backtrace(int sockfd, int type, void *ev){
 	case ARM_EVENT:
 		rc = set_arm_event_backtrace(buf, (struct ras_arm_event *)ev);
 		break;
+	case DEVLINK_EVENT:
+		rc = set_devlink_event_backtrace(buf, (struct devlink_event *)ev);
+		break;
+	case DISKERROR_EVENT:
+		rc = set_diskerror_event_backtrace(buf, (struct diskerror_event *)ev);
+		break;
+	case MF_EVENT:
+		rc = set_mf_event_backtrace(buf, (struct ras_mf_event *)ev);
+		break;
 	default:
 		return -1;
 	}
@@ -349,7 +434,7 @@ int ras_report_mc_event(struct ras_events *ras, struct ras_mc_event *ev){
 
 mc_fail:
 
-	if(sockfd > 0){
+	if(sockfd >= 0){
 		close(sockfd);
 	}
 
@@ -399,7 +484,7 @@ int ras_report_aer_event(struct ras_events *ras, struct ras_aer_event *ev){
 
 aer_fail:
 
-	if(sockfd > 0){
+	if(sockfd >= 0){
 		close(sockfd);
 	}
 
@@ -448,7 +533,7 @@ int ras_report_non_standard_event(struct ras_events *ras, struct ras_non_standar
 
 non_standard_fail:
 
-	if(sockfd > 0){
+	if(sockfd >= 0){
 		close(sockfd);
 	}
 
@@ -493,7 +578,7 @@ int ras_report_arm_event(struct ras_events *ras, struct ras_arm_event *ev){
 
 arm_fail:
 
-	if(sockfd > 0){
+	if(sockfd >= 0){
 		close(sockfd);
 	}
 
@@ -539,7 +624,7 @@ int ras_report_mce_event(struct ras_events *ras, struct mce_event *ev){
 
 mce_fail:
 
-	if(sockfd > 0){
+	if(sockfd >= 0){
 		close(sockfd);
 	}
 
@@ -549,3 +634,145 @@ mce_fail:
 		return -1;
 	}
 }
+
+int ras_report_devlink_event(struct ras_events *ras, struct devlink_event *ev){
+	char buf[MAX_MESSAGE_SIZE];
+	int sockfd = 0;
+	int done = 0;
+	int rc = -1;
+
+	memset(buf, 0, sizeof(buf));
+
+	sockfd = setup_report_socket();
+	if(sockfd < 0){
+		return -1;
+	}
+
+	rc = commit_report_basic(sockfd);
+	if(rc < 0){
+		goto devlink_fail;
+	}
+
+	rc = commit_report_backtrace(sockfd, DEVLINK_EVENT, ev);
+	if(rc < 0){
+		goto devlink_fail;
+	}
+
+	sprintf(buf, "ANALYZER=%s", "rasdaemon-devlink");
+	rc = write(sockfd, buf, strlen(buf) + 1);
+	if(rc < strlen(buf) + 1){
+		goto devlink_fail;
+	}
+
+	sprintf(buf, "REASON=%s", "devlink health report problem");
+	rc = write(sockfd, buf, strlen(buf) + 1);
+	if(rc < strlen(buf) + 1){
+		goto devlink_fail;
+	}
+
+	done = 1;
+
+devlink_fail:
+
+	if(sockfd >= 0){
+		close(sockfd);
+	}
+
+	if(done){
+		return 0;
+	}else{
+		return -1;
+	}
+}
+
+int ras_report_diskerror_event(struct ras_events *ras, struct diskerror_event *ev){
+	char buf[MAX_MESSAGE_SIZE];
+	int sockfd = 0;
+	int done = 0;
+	int rc = -1;
+
+	memset(buf, 0, sizeof(buf));
+
+	sockfd = setup_report_socket();
+	if(sockfd < 0){
+		return -1;
+	}
+
+	rc = commit_report_basic(sockfd);
+	if(rc < 0){
+		goto diskerror_fail;
+	}
+
+	rc = commit_report_backtrace(sockfd, DISKERROR_EVENT, ev);
+	if(rc < 0){
+		goto diskerror_fail;
+	}
+
+	sprintf(buf, "ANALYZER=%s", "rasdaemon-diskerror");
+	rc = write(sockfd, buf, strlen(buf) + 1);
+	if(rc < strlen(buf) + 1){
+		goto diskerror_fail;
+	}
+
+	sprintf(buf, "REASON=%s", "disk I/O error");
+	rc = write(sockfd, buf, strlen(buf) + 1);
+	if(rc < strlen(buf) + 1){
+		goto diskerror_fail;
+	}
+
+	done = 1;
+
+diskerror_fail:
+	if(sockfd >= 0){
+		close(sockfd);
+	}
+
+	if(done){
+		return 0;
+	}else{
+		return -1;
+	}
+}
+
+int ras_report_mf_event(struct ras_events *ras, struct ras_mf_event *ev)
+{
+	char buf[MAX_MESSAGE_SIZE];
+	int sockfd = 0;
+	int done = 0;
+	int rc = -1;
+
+	memset(buf, 0, sizeof(buf));
+
+	sockfd = setup_report_socket();
+	if (sockfd < 0)
+		return -1;
+
+	rc = commit_report_basic(sockfd);
+	if (rc < 0)
+		goto mf_fail;
+
+	rc = commit_report_backtrace(sockfd, MF_EVENT, ev);
+	if (rc < 0)
+		goto mf_fail;
+
+	sprintf(buf, "ANALYZER=%s", "rasdaemon-memory_failure");
+	rc = write(sockfd, buf, strlen(buf) + 1);
+	if (rc < strlen(buf) + 1)
+		goto mf_fail;
+
+	sprintf(buf, "REASON=%s", "memory failure problem");
+	rc = write(sockfd, buf, strlen(buf) + 1);
+	if (rc < strlen(buf) + 1)
+		goto mf_fail;
+
+	done = 1;
+
+mf_fail:
+	if (sockfd >= 0)
+		close(sockfd);
+
+	if (done)
+		return 0;
+	else
+		return -1;
+}
